#include <iostream>
using namespace std;
// The template class for implementing polynomials:
template<int power> class Polynom{
private:
   // The private field is an array with coefficients.
   // The size of the array is greater by 1 than
   // the power of the polynomial:
   double a[power+1];
public:
   // The constructor without arguments:
   Polynom(){
      // Fills the array:
      for(int k=0;k<=power;k++){
         a[k]=0;
      }
   }
   // The constructor with one argument:
   Polynom(double* nums){
      // Makes a copy:
      for(int k=0;k<=power;k++){
         a[k]=nums[k];
      }
   }
   // The method prints the elements of the array:
   void getAll(){
      cout<<"| ";
      for(int k=0;k<=power;k++){
         cout<<a[k]<<" | ";
      }
      cout<<endl;
   }
   // The operator method for calling the object:
   double operator()(double x){
      double s=0,q=1;
      for(int k=0;k<=power;k++){
         s+=a[k]*q;
         q*=x;
      }
      return s;
   }
   // The operator method for indexing the object:
   double &operator[](int k){
      return a[k];
   }
   // The template operator method for calculating
   // the product of polynomials:
   template<int n> Polynom<power+n> 
      operator*(Polynom<n> pol){
      // A local object of the template class:
      Polynom<power+n> tmp;
      // Calculates the coefficients
      // for the polynomial,
      // which is the result of the product:
      for(int i=0;i<=power;i++){
         for(int j=0;j<=n;j++){
            tmp[i+j]+=a[i]*pol[j];
         }
      }
      // The result of the method:
      return tmp;
   }
   // The template operator method for calculating
   // the sum of polynomials:
   template<int n> Polynom<(n>power?n:power)>
   operator+(Polynom<n> pol){
      int i;
      // A local object of the template class:
      Polynom<(n>power?n:power)> tmp;
      // Calculates the coefficients
      // for the polynomial,
      // which is the result of the method:
      for(i=0;i<=power;i++){
         tmp[i]+=a[i];
      }
      for(i=0;i<=n;i++){
         tmp[i]+=pol[i];
      }
      // The result of the method:
      return tmp;
   }
};
// The template operator function for calculating the
// result of multiplying a polynomial by a number:
template<int power> Polynom<power>
operator*(Polynom<power> pol,double r){
   // A local object of the template class:
   Polynom<power> tmp;
   // Calculates the coefficients for the polynomial,
   // which is the result of the function:
   for(int k=0;k<=power;k++){
      tmp[k]=pol[k]*r;
   }
   // The result of the function:
   return tmp;
}
// The template operator function for calculating the
// result of multiplying a number by a polynomial:
template<int power> Polynom<power>
operator*(double r,Polynom<power> pol){
   // The polynomial is multiplied by the number:
   return pol*r;
}
// The template operator function for calculating
// the difference of two polynomials:
template<int m,int n> Polynom<(m>n?m:n)>
operator-(Polynom<m> x,Polynom<n> y){
   // The first polynomial is added
   // the second polynomial multiplied by -1:
   return x+(-1)*y;
}
// The template function for calculating
// the derivative for a polynomial:
template<int power> Polynom<power-1>
Diff(Polynom<power> pol){
   // A local object of the template class:
   Polynom<power-1> tmp;
   // Calculates the coefficients for the polynomial,
   // which is the result of the function:
   for(int k=0;k<=power-1;k++){
      tmp[k]=pol[k+1]*(k+1);
   }
   // The result of the function:
   return tmp;
}
// The main function of the program:
int main(){
   // Arrays with the coefficients:
   double A[]={1,2,-1,1};
   double B[]={-1,3,0,2,-1,1};
   // The argument for the polynomial:
   double x=2;
   // The first polynomial:
   Polynom<3> P(A);
   cout<<"The polynomial P:\t";
   // The coefficients of the first polynomial:
   P.getAll();
   cout<<"The value P("<<x<<") = ";
   // The value of the first polynomial at the point:
   cout<<P(x)<<endl;
   cout<<"The polynomial P':\t";
   // The coefficients of the derivative
   // for the polynomial:
   Diff(P).getAll();
   cout<<"The value P'("<<x<<") = ";
   // The value of the derivative at the point:
   cout<<Diff(P)(x)<<endl;
   // The second polynomial:
   Polynom<5> Q(B);
   cout<<"The polynomial Q:\t";
   // The coefficients of the second polynomial:
   Q.getAll();
   cout<<"The value Q("<<x<<") = ";
   // The value of the second polynomial at the point:
   cout<<Q(x)<<endl;
   cout<<"The polynomial P*Q:\t";
   // The coefficients of the product
   // of the polynomials:
   (P*Q).getAll();
   cout<<"The value (P*Q)("<<x<<") = ";
   // The value of the polynomial product
   // at the point:
   cout<<(P*Q)(x)<<endl;
   cout<<"The polynomial P+Q:\t";
   // The coefficients of the polynomial sum:
   (P+Q).getAll();
   cout<<"The value (P+Q)("<<x<<") = ";
   // The value of the polynomial sum at the point:
   cout<<(P+Q)(x)<<endl;
   cout<<"The polynomial Q-P:\t";
   // The coefficients of the polynomial difference:
   (Q-P).getAll();
   cout<<"The value (Q-P)("<<x<<") = ";
   // The value of the polynomial difference
   // at the point:
   cout<<(Q-P)(x)<<endl;
   return 0;
}