#include <iostream>
#include <valarray>
using namespace std;
// The class to implement polynomials:
class Polynom{
private:
   // The private field is the array implemented
   // through an object of the class valarray:
   valarray<double> a;
public:
   // The constructor with an integer argument:
   Polynom(int n=0){
      // Defines the size of the array
      // and fills it with zeros:
      a.resize(n+1,0);
   }
   // The constructor whose arguments are the name of
   // a numeric array and the power of a polynomial:
   Polynom(double* nums,int n){
      // Creates a new object for the array:
      valarray<double> b(nums,n+1);
      // Assigns a new value to the field:
      a=b;
   }
   // The result of the method is the power
   // of the polynomial:
   int power(){
      return a.size()-1;
   }
   // The method prints the coefficients
   // of the polynomial:
   void getAll(){
      cout<<"| ";
      for(int k=0;k<=power();k++){
         cout<<a[k]<<" | ";
      }
      cout<<endl;
   }
   // The operator method allows calling the object.
   // The result is the value of the polynomial:
   double operator()(double x){
      // A copy of the array with the coefficients
      // of the polynomial:
      valarray<double> b(a);
      double q=1;
      // Calculates the elements of the array:
      for(int k=0;k<b.size();k++){
         b[k]*=q;
         q*=x;
      }
      // The result of the method is the sum
      // of the array elements:
      return b.sum();
   }
   // The operator method for indexing the objects:
   double &operator[](int k){
      return a[k];
   }
   // The operator method calculates the product
   // of polynomials:
   Polynom operator*(Polynom pol){
      // A local object:
      Polynom tmp(pol.power()+power());
      // Calculates the coefficients for
      // the resulting polynomial:
      for(int i=0;i<=power();i++){
         for(int j=0;j<=pol.power();j++){
            tmp[i+j]+=a[i]*pol[j];
         }
      }
      // The result of the method:
      return tmp;
   }
   // The operator method calculates the sum
   // of polynomials:
   Polynom operator+(Polynom pol){
      int i;
      int length=pol.power()>power()?pol.power():power();
      // The local object:
      Polynom tmp(length);
      // Calculates the coefficients for
      // the resulting polynomial:
      for(i=0;i<=power();i++){
         tmp[i]+=a[i];
      }
      for(i=0;i<=pol.power();i++){
         tmp[i]+=pol[i];
      }
      // The result of the method:
      return tmp;
   }
};
// The operator function for multiplying
// a polynomial by a number:
Polynom operator*(Polynom pol,double r){
   // A local object:
   Polynom tmp(pol.power());
   // Calculates the coefficients for
   // the resulting polynomial:
   for(int k=0;k<=pol.power();k++){
      tmp[k]=pol[k]*r;
   }
   // The result of the function:
   return tmp;
}
// The operator function for multiplying
// a number by a polynomial:
Polynom operator*(double r,Polynom pol){
   // The polynomial is multiplied by the number:
   return pol*r;
}
// The operator function for calculating
// the difference of two polynomials:
Polynom operator-(Polynom x,Polynom y){
   // The second polynomial is multiplied by -1
   // and added to the first polynomial:
   return x+(-1)*y;
}
// The template function for calculating
// the derivative from a polynomial:
Polynom Diff(Polynom pol){
   // A local object:
   Polynom tmp(pol.power()-1);
   // Calculates the coefficients for the
   // resulting polynomial:
   for(int k=0;k<=tmp.power();k++){
      tmp[k]=pol[k+1]*(k+1);
   }
   // The result of the function:
   return tmp;
}
// The main function of the program:
int main(){
   // Arrays with coefficients:
   double A[]={1,2,-1,1};
   double B[]={-1,3,0,2,-1,1};
   // The argument for the polynomials:
   double x=2;
   // The object for saving the results
   // of the calculations:
   Polynom res;
   // The first polynomial:
   Polynom P(A,3);
   cout<<"The polynomial P:\t";
   // The coefficients for the first polynomial:
   P.getAll();
   cout<<"The value P("<<x<<") = ";
   // The value of the first polynomial at the point:
   cout<<P(x)<<endl;
   // The derivative:
   res=Diff(P);
   cout<<"The polynomial P':\t";
   // The coefficients for the derivative
   // from the polynomial:
   res.getAll();
   cout<<"The value P'("<<x<<") = ";
   // The value of the derivative at the point:
   cout<<res(x)<<endl;
   // The second polynomial:
   Polynom Q(B,5);
   cout<<"The polynomial Q:\t";
   // The coefficients for the second polynomial:
   Q.getAll();
   cout<<"The value Q("<<x<<") = ";
   // The value of the second polynomial at the point:
   cout<<Q(x)<<endl;
   // The product of the polynomials:
   res=P*Q;
   cout<<"The polynomial P*Q:\t";
   // The coefficients for the product:
   res.getAll();
   cout<<"The value (P*Q)("<<x<<") = ";
   // The value of the product at the point:
   cout<<res(x)<<endl;
   // The sum of the polynomials:
   res=P+Q;
   cout<<"The polynomial P+Q:\t";
   // The coefficients for the sum:
   res.getAll();
   cout<<"The value (P+Q)("<<x<<") = ";
   // The value of the sum at the point:
   cout<<res(x)<<endl;
   // The difference of the polynomials:
   res=Q-P;
   cout<<"The polynomial Q-P:\t";
   // The coefficients for the difference:
   res.getAll();
   cout<<"The value (Q-P)("<<x<<") = ";
   // The value of the difference at the point:
   cout<<res(x)<<endl;
   return 0;
}