#include <iostream>
#include <cmath>
#include <string>
using namespace std;
// The function for solving equations
// by the bisection method:
double findRoot(double (*f)(double),double a,
double b,double dx){
   // The variable for saving the root:
   double x=(a+b)/2;
   // Calculates the root:
   while((b-a)/2>dx){
      // If the root is at the left boundary:
      if(f(a)==0){
         return a;
      }
      // If the root is at the right boundary:
      if(f(b)==0){
         return b;
      }
      // If the root is at the center of the interval:
      if(f(x)==0){
         return x;
      }
      // Moves a boundary to the center
      // of the interval:
      if(f(a)*f(x)>0){
         a=x;
      }
      else{
         b=x;
      }
      // The new value for the central point:
      x=(a+b)/2;
   }
   // The result of the function:
   return x;
}
// The functions determine the equations:
double f(double x){
   return 0.5*cos(x)-x;
}
double g(double x){
   return exp(-x)-x;
}
double h(double x){
   return x*x-5*x+6;
}
// The function solves equations
// and testes the roots:
void test(double (*f)(double),double a,double b,
string eq){
   // The monitored code:
   try{
      // If the function has values
      // of different signs at the boundaries:
      if(f(a)*f(b)>0){
         // Throws an error with a string value:
         throw "The interval is wrong!";
      }
      // The precision for the root:
      double dx=0.001;
      // The variable to save the root:
      double z;
      cout<<"The root of the equation "<<eq<<":\t";
      // The root of the equation:
      z=findRoot(f,a,b,dx);
      // Prints the result:
      cout<<z<<endl;
      cout<<"Testing the root:\t";
      // Testes the root:
      cout<<f(z)<<" = 0"<<endl;
   }
   // Catches the error:
   catch(const char* e){
      // Prints a message:
      cout<<e<<endl;
   }
   for(int k=1;k<=50;k++){
      cout<<"-";
   }
   cout<<endl;
}
// The main function of the program:
int main(){
   // Solves the equations:
   test(f,0,1,"0.5cos(x)-x=0");
   test(g,0,2,"exp(-x)-x");
   test(h,0,5,"x*x-5*x+6=0");
   test(h,0,2,"x*x-5*x+6=0");
   test(h,1,3,"x*x-5*x+6=0");
   test(h,2.5,4.5,"x*x-5*x+6=0");
   test(h,2.5,10,"x*x-5*x+6=0");
   return 0;
}