#include <iostream>
#include <cmath>
using namespace std;
// The function determines the differential equation:
double f(double x,double y){
   return x*x*exp(-x)-y;
}
// The function solves a differential equation
// by the classical Runge-Kutta method:
double dsolve(double (*f)(double,double),double x0,
double y0,double x){
   // The number of intervals:
   int n=1000;
   // The variables to save the values
   // which are used in the calculations:
   double p1,p2,p3,p4;
   // The increment for the argument:
   double h=(x-x0)/n;
   // The initial value of the unknown function:
   double y=y0;
   // Calculates the value of the unknown function
   // at the point:
   for(int k=0;k<n;k++){
      p1=f(x0+k*h,y);
      p2=f(x0+k*h+h/2,y+h*p1/2);
      p3=f(x0+k*h+h/2,y+h*p2/2);
      p4=f(x0+k*h+h,y+h*p3);
      y=y+(h/6)*(p1+2*p2+2*p3+p4);
   }
   // The value of the unknown function at the point:
   return y;
}
// The function determines the analytical solution
// of the differential equation:
double Y(double x){
   return (x*x*x/3+1)*exp(-x);
}
// The main function of the program:
int main(){
   // The array of the points at which
   // the solution of the differential equation
   // is calculated:
   double x[]={0,0.5,1,3,10};
   cout<<
   "The solution of the differential equation:\n";
   for(int k=0;k<5;k++){
      cout<<dsolve(f,0,1,x[k])<<" vs. "<<
      Y(x[k])<<endl;
   }
   return 0;
}